function [xt, ft, succ, numf, iarm, lambda, lamc] = armijo_search_mod(prob, xc,fc,gc,dir,lambda, alp)

% Armijo line search with polynomial interpolation
% Adapted from steep by C. T. Kelley, Dec 20, 1996

% xc, current point
% fc = F(sys,xc)
% gc is gradient
% dir is search direction
%
% returns: xt, ft: Armijo point and F(sys,xt)
%          succ = true if success
%          numf: nb of F evals
%          iarm: nb of Armijo backtrackings

% Modified by BM


MAX_IARM = 50;

numf = 0;

% trial evaluation at full step
qp0 = ip(xc,gc,dir); 
lambda = -sign(qp0) * abs(lambda); % Sign correction
lamc = lambda;  % Guess stepsize

xt = moveEIG(prob,xc,dir,lambda);
ft = F(prob,xt); % new cost 
numf = numf+1; iarm = 0;

armijo = (fc - ft) >= alp*abs(lambda*qp0);

while ~armijo  
    iarm = iarm + 1;
    lambda = lambda / 2;
    xt = moveEIG(prob,xc,dir,lambda);
    ft = F(prob, xt);
    numf = numf + 1; 
    if(iarm > MAX_IARM)
        succ = false;        
        return
    end    
    armijo = (fc - ft) >= alp*abs(lambda*qp0);
end
succ = true;
end
