function [x,stats,fail] = rtr_rank_ubv(prob,opts,x)
% Newton + TR 

% About: RTR - Riemannian Trust-Region
% (C) 2004-2007, P.-A. Absil, C. G. Baker, K. A. Gallivan
% Florida State University
% School of Computational Science

   % set pointers for functions required by RTR
   n = prob.n1;
   m = prob.n2;
   k = prob.k;

   fns.R    = @(x,h)retract_ubv(prob,x,h,1);      % retraction
   fns.g    = @(x,h1,h2)metric_ubv(x,h1,h2);      % Riemannian metric
   fns.proj = @(x,z)lift_ubv(x,z);   % projection onto horizontal plane from nearby
   fns.f     = @(x)cost_ubv(prob,x);     % objective function
   fns.fgrad = @(x)grad_ubv(prob,x);  % gradient of f
   fns.fhess = @(x,eta)hess_ubv(prob,x,eta);     % Hessian of f
   fns.randT = @(x)randHxM_ubv(x);
   fns.scale = @(h,c)scaleHxM_ubv(h,c);
   fns.plus  = @(h1,h2,c1,c2)plusHxM_ubv(h1,h2,c1,c2);   
   fns.prob  = @()(prob);
   fns.normX = @(x) normX_ubv(x);
   if opts.guess_Delta,
       fns.guess_linesearch = @(x, errors, fgradx1, data_ls, max_step)guess_linesearch_ubv(x, errors, fgradx1, data_ls);
   end
   
   %fns.shrink_rank = @(x, fx)shrink_rank(prob, x, fx);
   %fns.prec   = @(x,eta)invHessVec_direct_bicgstab(prob,x,eta);
   % set parameters for RTR algorithm
   
   
   params.x0 = x;
 
    if ~isfield(x, 'Delta0'),
       x.Delta0 = k;
   end
 
   %Maximum trust-region radius
   if ~isfield(params,'Delta_bar')
     %params.Delta_bar = m * k; % Bart
     %params.Delta_bar = 10*k; % original
     params.Delta_bar = (2^10)*x.Delta0; % trying

   end
   
   %Initial trust-region radius
   if ~isfield(params,'Delta0'),
      %params.Delta0   = params.Delta_bar/8; % Bart
%       params.Delta0 = k; % original
      params.Delta0 = x.Delta0;

   end
   
   if ~isfield(params,'epsilon')
       params.epsilon = 0;%1e-6;
   end
   if ~isfield(params,'max_outer')
       params.max_outer = opts.maxit;
   end
   
   if ~isfield(params,'max_inner')
       params.max_inner = opts.max_inner; %k*(2*n-k);
   end
   %params.max_inner = 1;
   params.verbosity = opts.verbosity;
   t_ubv = tic;
   [x,stats,fail,infos] = rtr_quotient_ubv(fns,params,prob,opts);
   toc(t_ubv)
   x.infos = infos;
 
   
   
