function[xmin,  cost_new, errors_new] = guess_linesearch_right_inv_gh(model, errors, searchDir, data_ls)
% [xmin, cost_new, errors_new] = guess_linesearch_right_inv_gh(model, errors,
% searchDir, data_ls)
%
%
% Computes the optimal stepsize in the direction "searchDir" at the iterate
% "model".
%
% Parameters:
% -----------
% model: Structured array with current low-rank factors
% errors: Array of the discrepancies between given entries and the predicted
%         values at the current iterate
% searchDir: Sturctured array with the search directions
% data_ls: Structured array related to given entries
%
% Output:
% -------
% xmin: Optimal stepsize
% cost_new: Cost at the next iterate
% errors_new: Array of the discrepancies between given entries and the predicted
%             values at the new iterate
%
% This is an algorithmic implementation of
% "A Riemannian geometry for low-rank matrix completion"
% B. Mishra, K. Adithya Apuroop and R. Sepulchre
% Technical report (arXiv:1211.1550), 2012
%
% Author:
% Bamdev Mishra, <b.mishra@ulg.ac.be>, 2012

n = data_ls.nentries;

s_star = partXY([model.G    searchDir.G ]', [searchDir.H  model.H ]', data_ls.rows, data_ls.cols, n); %row
s_star_gh = partXY(searchDir.G', searchDir.H', data_ls.rows, data_ls.cols, n); % row


% A degree 4 polynomial
p =  (s_star_gh*s_star_gh');%   sum(s_star_gh.^2); %norm(s_star_gh)^2;
q = 2*(s_star_gh*s_star'); %  2*sum(s_star_gh .* s_star);
r = 2*(s_star_gh*errors) + (s_star*s_star');%     2*sum(s_star_gh .* errors) + sum(s_star.^2);
s = 2*(s_star*errors); %2*sum(errors .* s_star);
t =  (errors'*errors); % sum(errors.^2); % norm(errors')^2;
% old_cost = t / n % old cost


% f = @(x)p*x.^4 + q*x.^3 + r*x.^2 + s*x.^1 + t;
% xmin = fminbnd(f,  max_step / 16, 16*max_step)

coefficients_poly = [p, q, r, s, t];
coefficients = [4*p , 3*q, 2*r, s]; % Coefficient of the derivative polynomial
xs = roots(coefficients); % Roots of the degree 3 polynomial

% Extracting the real roots
% real_roots = [];
% for ii = 1 : 3
%     if isreal(xs(ii))
%         real_roots = [real_roots; xs(ii)];
%     end
% end
real_roots = xs(imag(xs)==0);
real_roots = 0.5*(real_roots + abs(real_roots));


% Finding the global minimum
% fun_val = [];
% for jj = 1 : length(real_roots)
%     x = real_roots(jj);
%     fun_val = [fun_val ; p*x^4 + q*x^3 + r*x^2 + s*x + t ];
% end
fun_val = polyval(coefficients_poly, real_roots);

[fmin, idx] = min(fun_val);

xmin = real_roots(idx);  % Global minimum
% cost_new = fmin /n;
cost_new = fmin /2;


errors_new = (errors')+(xmin*s_star)+((xmin^2)*s_star_gh); % row
errors_new = errors_new'; % column


return;
