function [Hess] = hess_gh(prob, x, h)
% Applies the hessian operator using the Riemannian correction derived in 
% arXiv:1211.1550, 2012.

  s_star_single_proj = partXY([h.G x.G]', [x.H h.H]', prob.Omega_i, prob.Omega_j, length(prob.Omega_i))';

  updateSparse(prob.temp_omega, s_star_single_proj);
  S_star = prob.temp_omega;

  Hess.G = S_star*(x.H*x.invHtH);
  Hess.H = S_star'*(x.G*x.invGtG);

  
  if isempty(x.cache.S)
      d = (x.on_omega - prob.data);
      updateSparse(prob.temp_omega, d);
      prob.temp_omega = prob.temp_omega;
      S = prob.temp_omega;
  else
      S = x.cache.S;
  end

  
  if isempty(x.cache.SH)
      SH = S*x.H;
      SG = S'*x.G;
  else
      SH = x.cache.SH;
      SG = x.cache.SG;
  end
  
  ShH = S*h.H;
  ShG = S'*h.G;
  gradG = x.gradG;
  gradH = x.gradH;
  GgradG = x.GgradG;
  HgradH = x.HgradH;
  hHH = h.H'*x.H;
  hGG = h.G'*x.G;
  
  
   
  Hess.G = Hess.G + ShH*(x.invHtH) - 2*SH*(x.invHtH*symm(hHH)*x.invHtH);
  Hess.H = Hess.H + ShG*(x.invGtG) - 2*SG*(x.invGtG*symm(hGG)*x.invGtG);
  
  % We still need a correction factor for the non-constant metric
  Hess.G = Hess.G + gradG*(symm(hHH)*x.invHtH) + h.G*(symm(HgradH)*x.invHtH) - x.G*(symm(h.H'*gradH)*x.invHtH);
  Hess.H = Hess.H + gradH*(symm(hGG)*x.invGtG) + h.H*(symm(GgradG)*x.invGtG) - x.H*(symm(h.G'*gradG)*x.invGtG);
  
  
  % Projection on the Horizontal space
%   Hess = removeVxM_gh(x, Hess); % projection on to horizontal space
  
  Lambda =  (hHH) * x.invHtH  -   x.invGtG * (hGG);
  Lambda = Lambda/2;
  
  Hess.G = Hess.G + x.G*Lambda;
  Hess.H = Hess.H - x.H*Lambda';
    
end