function [x,stats,fail] = rtr_rank_gh(prob,opts,x)
% Newton + TR 

% About: RTR - Riemannian Trust-Region
% (C) 2004-2007, P.-A. Absil, C. G. Baker, K. A. Gallivan
% Florida State University
% School of Computational Science

   % set pointers for functions required by RTR
   n = prob.n1;
   m = prob.n2;
   k = prob.k;

   fns.R    = @(x,h)retract_gh(prob,x,h,1);      % retraction
   fns.g    = @(x,h1,h2)metric_gh(x,h1,h2);      % Riemannian metric
   fns.proj = @(x,z)lift_gh(x,z);   % projection onto horizontal plane from nearby
   fns.f     = @(x)cost_gh(prob,x);     % objective function
   fns.fgrad = @(x)grad_gh(prob,x);  % gradient of f
   fns.fhess = @(x,eta)hess_gh(prob,x,eta);     % Hessian of f
   fns.randT = @(x)randHxM_gh(x);
   fns.scale = @(h,c)scaleHxM_gh(h,c);
   fns.plus  = @(h1,h2,c1,c2)plusHxM_gh(h1,h2,c1,c2);   
   fns.prob  = @()(prob);
   fns.normX = @(x) normX_gh(x);
   if opts.guess_Delta,
       fns.guess_linesearch = @(x, errors, fgradx1, data_ls, max_step)guess_linesearch_gh(x, errors, fgradx1, data_ls);
   end
   %fns.shrink_rank = @(x, fx)shrink_rank(prob, x, fx);
   %fns.prec   = @(x,eta)invHessVec_direct_bicgstab(prob,x,eta);
   % set parameters for RTR algorithm
   
   
   params.x0 = x;
   
   if ~isfield(x, 'Delta0'),
       x.Delta0 = min(n, m)*k/8;
   end
 
   %Maximum trust-region radius
   if ~isfield(params,'Delta_bar')
     %params.Delta_bar = m * k; % Bart
     %params.Delta_bar = 10*k; % original
%     params.Delta_bar = (2^round(log2(k)))*min(n, m); % trying
%       params.Delta_bar = k * min(n, m); % trying
      params.Delta_bar = (2^10) * x.Delta0;

   end
   
   %Initial trust-region radius
   if ~isfield(params,'Delta0'),
      %params.Delta0   = params.Delta_bar/8; % Bart
      %params.Delta0 = k; % original
%       params.Delta0 = params.Delta_bar / 8; % trying
      params.Delta0 = x.Delta0;

   end
   
   if ~isfield(params,'epsilon')
       params.epsilon = 0;%1e-6;
   end
   if ~isfield(params,'max_outer')
       params.max_outer = opts.maxit;
   end
   
   if ~isfield(params,'max_inner')
       params.max_inner = opts.max_inner;
   end
   %params.max_inner = 1;
   params.verbosity = opts.verbosity;
   
   [x,stats,fail,infos] = rtr_quotient(fns,params,prob,opts);
   
   x.infos = infos;
 
   
   
